@@ -1,5 +1,5 @@
 # This file is part of Pebble.
-# Copyright (c) 2013-2021, Matteo Cafasso
+# Copyright (c) 2013-2022, Matteo Cafasso
 
 # Pebble is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License
@@ -75,7 +75,8 @@ def process(*args, **kwargs):
 
 
 def _process_wrapper(function, timeout, name, daemon, mp_context):
-    _register_function(function)
+    if isinstance(function, types.FunctionType):
+        _register_function(function)
 
     if hasattr(mp_context, 'get_start_method'):
         start_method = mp_context.get_start_method()
@@ -87,9 +88,9 @@ def _process_wrapper(function, timeout, name, daemon, mp_context):
         future = ProcessFuture()
         reader, writer = mp_context.Pipe(duplex=False)
 
-        if start_method != 'fork':
+        if isinstance(function, types.FunctionType) and start_method != 'fork':
             target = _trampoline
-            args = [_qualname(function), function.__module__] + list(args)
+            args = [function.__qualname__, function.__module__] + list(args)
         else:
             target = function
 
@@ -117,6 +118,9 @@ def _worker_handler(future, worker, pipe, timeout):
     """
     result = _get_result(future, pipe, timeout)
 
+    if worker.is_alive():
+        stop_process(worker)
+
     if isinstance(result, BaseException):
         if isinstance(result, ProcessExpired):
             result.exitcode = worker.exitcode
@@ -125,9 +129,6 @@ def _worker_handler(future, worker, pipe, timeout):
     else:
         future.set_result(result)
 
-    if worker.is_alive():
-        stop_process(worker)
-
 
 def _function_handler(function, args, kwargs, pipe):
     """Runs the actual function in separate process and returns its result."""
@@ -166,9 +167,6 @@ def _validate_parameters(timeout, name, daemon, mp_context):
         raise TypeError('Name expected to be None or string')
     if daemon is not None and not isinstance(daemon, bool):
         raise TypeError('Daemon expected to be None or bool')
-    if mp_context is not None and not isinstance(
-            mp_context, multiprocessing.context.BaseContext):
-        raise TypeError('Context expected to be None or multiprocessing.context')
 
 
 ################################################################################
@@ -179,7 +177,7 @@ _registered_functions = {}
 
 
 def _register_function(function):
-    _registered_functions[_qualname(function)] = function
+    _registered_functions[function.__qualname__] = function
 
     return function
 
@@ -212,14 +210,3 @@ def _function_lookup(name, module):
             return _registered_functions[name]
         except KeyError:  # decorator without @pie syntax
             return _register_function(function)
-
-
-def _qualname(function):
-    """Returns the fully qualified domain of a function."""
-    try:
-        return function.__qualname__
-    except AttributeError:  # Python 2
-        if isinstance(function, types.MethodType):
-            return '.'.join((function.im_class, function.__name__))
-
-        return function.__name__
